import * as THREE from "three";
import { 
alertAsync
,confirmAsync 
,choiceAsync
,promptAsync
} from "./dialogUtil";
import { 
geodata
,init_geodata
} from "./roomconf";
import { 
loadModel
,loadModelFromArrayBuffer
,loadPly
,loadPlyFromText
,makePlyGeometryFromText
,faceTo
} from "./modelUtil";

export async function createWorldObjects({
  scene,
  collisionObjects,
  _interactableObjects,
  teleportTargetObjects,
}) {
  
  //geodataから地形
  init_geodata();//
  let gd_idx = 0;
  for(const gd of geodata){
    if(gd.type=="box"){

      const box = new THREE.Mesh(
        new THREE.BoxGeometry(gd.sx, gd.sy, gd.sz),
        new THREE.MeshLambertMaterial({
          color: gd.color,
          side: THREE.DoubleSide,
        })
      );
      gd_idx++;
      if(gd.name){
        box.name = gd.name;
      }else{
        box.name = "box_"+gd_idx;
      }
      box.position.set(gd.px, gd.py, gd.pz);

      scene.add(box);

      //ロード時処理
      if(gd.onLoad){
        gd.onLoad(box);
      }


      //クリックイベントの設定
      if(gd.onSelectDown){
        box.onSelectDown = gd.onSelectDown;
        window.arr_interactive.push(box);
      }
      if(gd.collision=="wall"){
        //障害物
        collisionObjects.push(box);
      }else if(gd.collision=="none"){
        //見た目だけで判定なし
      }else{
        //移動可能地形
        teleportTargetObjects.push(box);
      }

    }else if(gd.type=="plane"){
      const obj  = new THREE.Mesh(
        new THREE.PlaneGeometry(gd.sx, gd.sz, 1, 1),
        new THREE.MeshLambertMaterial({
          color: gd.color,
        })
      );
      //xz平面のみ
      obj.rotation.x = Math.PI / -2;
      obj.receiveShadow = true;//影落とす
      gd_idx++;
      if(gd.name){
        obj.name = gd.name;
      }else{
        obj.name = "plane_"+gd_idx;
      }
      obj.position.set(gd.px, gd.py, gd.pz);
      scene.add(obj);

      //ロード時処理
      if(gd.onLoad){
        gd.onLoad(obj);
      }
      //クリックイベントの設定
      if(gd.onSelectDown){
        box.onSelectDown = gd.onSelectDown;
        window.arr_interactive.push(obj);
      }
      if(gd.collision=="wall"){
        //障害物
        collisionObjects.push(obj);
      }else if(gd.collision=="none"){
        //見た目だけで判定なし
      }else{
        //移動可能地形
        teleportTargetObjects.push(obj);
      }

    }else if(gd.type=="glb"){
      let _glb = await loadModel(gd.resource);
      const glb = _glb.scene;
      const obj = new THREE.Object3D();
      obj.add(glb);
      obj.actions = {};

      gd_idx++;
      if(gd.name){
        obj.name = gd.name;
      }else{
        obj.name = "glb_"+gd_idx;
      }


      //埋め込みアニメーション対応
      const animations = _glb.animations;
      if( animations && animations.length) {
        //Animation Mixerインスタンスを生成
        let mixer = new THREE.AnimationMixer(obj);
        //全てのAnimation Clipに対して
        for (let i = 0; i < animations.length; i++) {
            let animation = animations[i];
            //Animation Actionを生成
            let action = mixer.clipAction(animation) ;
            //アニメーションの最後のフレームでアニメーションが終了
            action.clampWhenFinished = true;
            //アニメーションを再生
            if(gd.animation =="auto"){
              action.play();
            }
            obj.actions[animation.name] = action;

        }
        //静的なので削除は考慮しない
        let anifn = function(dt){
          mixer.update(dt);
        }
        window.animation_ticks.push(anifn);
      }

      obj.scale.set(gd.sx, gd.sy, gd.sz);
      obj.position.set(gd.px, gd.py, gd.pz);
      obj.rotation.set(gd.rx, gd.ry, gd.rz);
      obj.traverse(
        (object) => { //モデルの構成要素
          if(object.isMesh) { //その構成要素がメッシュだったら
            object.castShadow = true;//影表示
            object.material.metalness = 0;//これしないと暗くなる
          }
        }
      );
      scene.add(obj);

      //ロード時処理
      if(gd.onLoad){
        gd.onLoad(obj);
      }


      //クリックイベントの設定
      if(gd.onSelectDown){
        obj.onSelectDown = gd.onSelectDown;
        window.arr_interactive.push(obj);
      }
      if(gd.collision=="wall"){
        //障害物
        collisionObjects.push(obj);
      }else if(gd.collision=="none"){
        //見た目だけで判定なし
      }else{
        //移動可能地形
        teleportTargetObjects.push(obj);
      }
    }else if(gd.type=="ply"){
      let obj = await loadPly(gd.resource);
      obj.scale.set(gd.sx, gd.sy, gd.sz);
      obj.position.set(gd.px, gd.py, gd.pz);
      obj.rotation.set(gd.rx, gd.ry, gd.rz);

      gd_idx++;
      if(gd.name){
        obj.name = gd.name;
      }else{
        obj.name = "glb_"+gd_idx;
      }
      
      scene.add(obj);


      //ロード時処理
      if(gd.onLoad){
        gd.onLoad(obj);
      }

      //クリックイベントの設定
      if(gd.onSelectDown){
        obj.onSelectDown = gd.onSelectDown;
        window.arr_interactive.push(obj);
      }
      if(gd.collision=="wall"){
        //障害物
        collisionObjects.push(obj);
      }else if(gd.collision=="none"){
        //見た目だけで判定なし
      }else{
        //移動可能地形
        teleportTargetObjects.push(obj);
      }
    }
  }

}
